-- PLBARS 1..10 reserved, [1] sw width, [2] sw height, [3] sw class height, [4] show level, [5] RGBA substrate, [6] update freq;
-- PLBARS[7] - show actionbar
-- PLBARS 11..20 - player1, 21..30 - player2, 31..40 - player3, 41..50 - player4, 51..60 - healthbar, 61 manabar, 71 actionbar,
-- 81 currbar
-- for each entry [10*i+1] left, [10*i+2] bottom; health/mana/cast bars: [10*i+3] width, [10*i+4] height, [10*i+5] color, 
-- curr bar [10*i+6] show / hide

local helpText = {
"|cFFffff00PlayerBars short help and command list:|r",
"|cFFffff00For full documentation please visit|r",
"|cFF00ffffhttp://www.curse.com/addons/wow/playerbars|r",
"Use Shift + Mouse Left Click to drag elements",
"|cFF00ffff/pbars|r - toggles all elements",
"|cFF00ffff/pbars hw <number>|r - sets health bar width",
"|cFF00ffff/pbars hh <number>|r - sets health bar height",
"|cFF00ffff/pbars hc <number> <number> <number> <number>|r",
"- sets health bar rgba color",
"- use |cFF00ffffmw|r, |cFF00ffffmh|r, |cFF00ffffmc|r instead of |cFF00ffffhw|r, |cFF00ffffhh|r, |cFF00ffffhc|r to change power bar",
"- use |cFF00ffffaw|r, |cFF00ffffah|r, |cFF00ffffac|r instead of |cFF00ffffhw|r, |cFF00ffffhh|r, |cFF00ffffhc|r to change cast bar",
"|cFF00ffff/pbars c <number> <number> <number> <number>|r",
"- sets health/mana/cast bar background rgba color",
"|cFFffff00Cast bar:|r",
"|cFF00ffff/pbars a|r - switches between default and custom cast bar",
"|cFFffff00Currency bar:|r",
"|cFF00ffff/pbars cb|r - toggles currency bar",
"|cFFffff00Group commands:|r",
"|cFF00ffff/pbars g|r - toggles group swatches",
"|cFF00ffff/pbars gw <number>|r - sets swatch width",
"|cFF00ffff/pbars gh <number>|r - sets swatch height",
"|cFFffff00Block commands:|r",
"|cFF00ffff/pbars b|r - justifies health/mana/cast bars and puts them together",
"|cFF00ffff/pbars bgh <number>|r - lines up swatches in a row separated by value",
"|cFF00ffff/pbars bgv <number>|r - lines up swatches in a column separated by value"
}

local timer = 0.0
local timer2 = 0.0
local ratio = {1, 1, 1, 1,  1, 1, 0}
local rleft, rbottom, rwidth, rheight
local currBar = {5, 1, 20} -- margin, gap, subbar height

local playerName = UnitName("player")

local curList = {823, 824, 1101}

local icons = {
	[1] = "Interface\\Icons\\inv_apexis_draenor",
	[2] = "Interface\\Icons\\inv_garrison_resource",
	[3] = "Interface\\Icons\\garrison_oil",
	[4] = "Interface\\Icons\\inv_misc_coin_01"
}

local xRGBA = {
	["high"] = {0, 1, 0, 0.5},
	["mid"] =  {1, 1, 0, 0.5},
	["low"] =  {1, 1, 0, 0.5},
	["crit"] = {1, 0, 0, 0.5},
	["zero"] = {0, 0, 0, 0.5},
	["off"] =  {0.6, 0.6, 0.6, 0.3},
	["dark"] = {0, 0, 0, 0.5},
	["pitch"] = {0, 0, 0, 0.66}
}

-- default bar colors
local barColor = {
	["HEALTH"] = {0,0.7,0,0.8},
	["MANA"] = {0,0,0.7,0.8},
	["RAGE"] = {0.7,0,0,0.8},
	["FOCUS"] = {0.6,0.4,0.3,0.8},
	["ENERGY"] = {0.7,0.7,0,0.8},
	["COMBO_POINTS"] = {0.5,0.5,0.5,0.8},
	["RUNES"] = {0.5,0.5,0.5,0.8},
	["RUNIC_POWER"] = {0.5,0.5,0.5,0.8},
	["SOUL_SHARDS"] = {0.5,0.5,0.5,0.8},
	["ECLIPSE"] = {0.5,0.5,0.5,0.8},
	["HOLY_POWER"] = {0.5,0.5,0.5,0.8},
	["AMMOSLOT"] = {1,0,0,0.8},
	["FUEL"] = {1,1,0,0.8},
	["CASTBAR"] = {1,0,1,0.8},
	["SUBSTR"] = {1,1,1,0.25}
}
local classPT = {"RAGE", "MANA", "FOCUS", "ENERGY", "MANA", "RUNIC_POWER", "MANA", "MANA", "MANA", "ENERGY", "MANA"}

-- 1..4 swatches, frame ref, text ref, class frame ref, party index
-- 5 healthbar, 6 manabar, 7 actionbar, 8 main cur bar, 81 - 84 sub bars
xRef = {}
for i = 1, 8 do
	xRef[i] = {}
end
for i = 81, 84 do
	xRef[i] = {}
end

local className = {"WR", "PL", "HT", "RG", "PR", "DK", "SH", "MG", "LK", "MN", "DR"}  
local roleName = {["DAMAGER"] = "D", ["HEALER"] = "H", ["NONE"] = "X", ["TANK"] = "T"}

local function GetGroupInfo()
    local unit, classInd, role, name, hRatio
	for i = 1, 4 do
		unit = "party" .. i
		if UnitExists(unit) then
			name = UnitName(unit)
			if name ~= playerName then
				-- DAMAGER, HEALER, NONE, TANK
				role = UnitGroupRolesAssigned(unit)
				_, _, classInd = UnitClass(unit)
				hRatio = UnitHealth(name) / UnitHealthMax(name)
				
				if not UnitIsConnected(name) or UnitHealth(name) < 2 then hRatio = 0 end
				
				if hRatio > 0.75 then
					xRef[i][2]:SetTexture(unpack(xRGBA["high"]))
				elseif hRatio > 0.5 then
					xRef[i][2]:SetTexture(unpack(xRGBA["mid"]))
				elseif hRatio > 0.25 then
					xRef[i][2]:SetTexture(unpack(xRGBA["low"]))
				elseif hRatio > 0 then
					xRef[i][2]:SetTexture(unpack(xRGBA["crit"]))
				else
					xRef[i][2]:SetTexture(unpack(xRGBA["zero"]))
				end
				
				xRef[i][1].text:SetText(roleName[role])
				if classInd ~= nill then
					xRef[i][3].text:SetText(className[classInd] .. "-" .. UnitLevel(unit))
				else
					xRef[i][3].text:SetText("??" .. "-" .. UnitLevel(unit))
				end
			end
		else
			xRef[i][2]:SetTexture(unpack(xRGBA["zero"]))
			xRef[i][1].text:SetText("-")
			xRef[i][3].text:SetText("-")
		end
	end
end

local function setActionBar()
	if PLBARS[7] == 1 then
		xRef[7][1]:Show()
		_G["CastingBarFrame"].showCastbar = false
	else
		xRef[7][1]:Hide()
		_G["CastingBarFrame"].showCastbar = true
	end
end

local function resetFrame(frRef, frPoint, frPar, parPoint, xx, yy)
		local left, bottom, _, height = frRef:GetBoundsRect()
		local x = xx or left
		local y = yy or bottom + height
		if frPar == nil then frPar = WorldFrame end
		if frPoint == nil then frPoint = "TOPLEFT" end
		if parPoint == nil then parPoint = "BOTTOMLEFT" end
		frRef:ClearAllPoints()
		frRef:SetPoint(frPoint, frPar, parPoint, x, y)
end

local function safeSetWidth(frRef, w)
	if w < 1 then w = 1 end
	frRef:SetWidth(w)
end

local function onUpdate(self,elapsed)

	-- curr bar update once a sec
	timer2 = timer2 + elapsed
	if xRef[8][1]:IsVisible() and timer2 > 1.0 then
		local currVal, money, gold, silver, copper
		for i = 1, 3 do
			currVal = select(2, GetCurrencyInfo(curList[i]))
			xRef[80 + i][5].text:SetText(tostring(currVal))
		end
		money = GetMoney()
		gold = floor(abs(money / 10000))
		silver = floor(abs(mod(money / 100, 100)))
		copper = floor(abs(mod(money, 100)))
		xRef[80 + 4][5].text:SetText(format("%d %d %d", gold, silver, copper))
		timer2 = 0
	end

	timer = timer + elapsed

	if timer > PLBARS[6] then
		ratio[5] = UnitHealth(playerName) / UnitHealthMax(playerName)
		ratio[6] = UnitPower(playerName) / UnitPowerMax(playerName)
		for i = 5, 6 do
			safeSetWidth(xRef[i][3], PLBARS[10*i + 3] * ratio[i])
		end

		-- castbar
		local CBname, CBtext, startTime, endTime
		ratio[7] = 0
		CBname, _, CBtext, _, startTime, endTime = UnitCastingInfo("player")
		if CBname ~= nil then
			ratio[7] = (GetTime()*1000 - startTime) / (endTime - startTime)
		else
			CBname, _, CBtext, _, startTime, endTime = UnitChannelInfo("player")
			if CBname ~= nil then
				ratio[7] = (GetTime()*1000 - startTime) / (endTime - startTime)
				CBtext = CBname
			end
		end
		if CBtext ~= nil then
			xRef[7][5].text:SetText(" " .. strupper(CBtext))
		else
			xRef[7][5].text:SetText(" ")
		end
		safeSetWidth(xRef[7][3], ratio[7] * PLBARS[10*7 + 3])
		
		-- group state
		if xRef[1][1]:IsVisible() then
			if IsInGroup() then 
				GetGroupInfo()
			else
				-- reset empty values
				for i = 1, 4 do
					xRef[i][2]:SetTexture(unpack(xRGBA["off"]))
					xRef[i][1].text:SetText(" ")
					xRef[i][3].text:SetText(" ")
				end
			end
		end
		
		timer = 0
	end
end

local function onMouseDown(self, button)
	if button == "LeftButton" and IsShiftKeyDown() and not self.isMoving then
		self:StartMoving();
		self.isMoving = true;
	end
end

local function onMouseUp(self, button)
	if button == "LeftButton" then
		if self.isMoving then
			self:StopMovingOrSizing();
			self.isMoving = false;
			-- 4 swatches left, bottom(inc frClass); 5,6,7 - health/mana/cast; 8 - curr
			for i = 1, 8 do
				PLBARS[10 * i + 1], PLBARS[10 * i + 2] = xRef[i][1]:GetBoundsRect()
			end
		end
	end
end

local function setFrameProps(frRef)
	frRef:SetMovable(true)
	frRef:EnableMouse(true)
	frRef:SetScript("OnMouseDown", onMouseDown)
	frRef:SetScript("OnMouseUp", onMouseUp)
end

local function createFrame(fname, x, y, w, h, RGBA, txt, overlay, r, img, parent, passive)
	local frSub, frSubTex,  frOver, frOverTex,  frTxt,  frIco, frIcoTex
	local hr, hg, hb, ha
	local txOff = 0
	local imgMar = 3
	local parent = parent or WorldFrame
	
	frSub = CreateFrame("Frame", fname, UIParent)
	frSub:SetClampedToScreen(true)
	frSub:SetSize(w, h) 
	frSub:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", x, y)
	frSubTex = frSub:CreateTexture()
	frSubTex:SetAllPoints()
	if overlay == nil then
		hr, hg, hb, ha = unpack(RGBA)
	else
		hr, hg, hb, ha = unpack(PLBARS[5])
	end
	frSubTex:SetTexture(hr, hg, hb, ha)
	frSub.background = frSubTex

	-- icon frame
	if img ~= nil then
		frIco = CreateFrame("Frame", "Icon", frSub)
		frIco:SetClampedToScreen(true)
		frIco:SetSize(h-imgMar*2, h-imgMar*2) 
		frIco:SetPoint("BOTTOMLEFT", frSub, "BOTTOMLEFT", imgMar, imgMar)
		frIcoTex = frIco:CreateTexture()
		frIcoTex:SetAllPoints()
		frIcoTex:SetTexture(img)
		frIco.background = frIcoTex
		txOff = h
	end

	if overlay ~= nil then
		frOver = CreateFrame("Frame", "SliderFrame", frSub)
		frOver:SetClampedToScreen(true)
		safeSetWidth(frOver, w*r)
		frOver:SetHeight(h) 
		frOver:SetPoint("BOTTOMLEFT", frSub, "BOTTOMLEFT", 0, 0)
		frOverTex = frOver:CreateTexture()
		frOverTex:SetAllPoints()
		frOverTex:SetTexture(unpack(RGBA))
		frOver.background = frOverTex
	end
	
	if txt ~= nil then
		frTxt = CreateFrame("Frame", "TextFrame", frSub)
		frTxt:SetClampedToScreen(true)
		frTxt:SetSize(w-txOff, h) 
		frTxt:SetPoint("BOTTOMLEFT", frSub, "BOTTOMLEFT", txOff, 0)
		
		frTxt.text = frTxt:CreateFontString(nil,"ARTWORK")
		frTxt.text:SetFontObject("Tooltip_Small")
		frTxt.text:SetTextColor(1, 1, 0, 0.75)
		frTxt.text:SetMaxLines(1)
		frTxt.text:SetJustifyH("LEFT")
		frTxt.text:SetText(txt)
		frTxt.text:SetAllPoints()
	end
	
	if passive == nil then setFrameProps(frSub) end
	
	return frSub, frSubTex,  frOver, frOverTex,  frTxt
end

local function createSwatch(fname, x, y, w, h, txt, txt2, RGBA)
	
	local frRef, texRef = createFrame(fname, x, y, w, h, RGBA)
	local frClass, frClassTex

	if txt ~= nil then
		frRef.text = frRef:CreateFontString(nil,"ARTWORK")
		frRef.text:SetFontObject("GameFontHighlightLarge")
		frRef.text:SetTextColor(xRGBA["dark"])
		frRef.text:SetText(txt)
		frRef.text:SetAllPoints()
	end
	
	frClass = CreateFrame("Frame", "ClassFrame", frRef)
	frClass:SetSize(w, PLBARS[3]) 
	frClass:SetPoint("BOTTOMLEFT", frRef, "BOTTOMLEFT", 0, -PLBARS[3])
	frClassTex = frClass:CreateTexture()
	frClassTex:SetAllPoints()
	frClassTex:SetTexture(0, 0, 0, 0.5) 
	frClass.background = frClassTex
	
	if txt ~= nil then
		frRef.text = frRef:CreateFontString(nil,"ARTWORK")
		frRef.text:SetFontObject("GameFontHighlightLarge")
		frRef.text:SetTextColor(1, 1, 1, 0.5)
		frRef.text:SetText(txt)
		frRef.text:SetAllPoints()
	end
	
	if txt2 ~= nil then
		frClass.text = frClass:CreateFontString(nil,"ARTWORK")
		frClass.text:SetFontObject("Tooltip_Small")
		frClass.text:SetTextColor(1, 1, 0, 0.5)
		frClass.text:SetText(txt2)
		frClass.text:SetAllPoints()
	end
	
	setFrameProps(frRef)
	
	return frRef, texRef, frClass
end

local eframe = CreateFrame("Frame", "EventFrame", UIParent)
eframe:RegisterEvent("ADDON_LOADED")
function eframe:OnEvent(event, arg)
if event == "ADDON_LOADED" and arg == "PlayerBars" then

	if PLBARS == nil then PLBARS = {} end
	
	-- width, height
	for i = 5, 6 do
		PLBARS[10*i + 3] = PLBARS[10*i + 3] or 265
		PLBARS[10*i + 4] = PLBARS[10*i + 4] or 6
	end
	PLBARS[10*7 + 3] = PLBARS[10*7 + 3] or 265
	PLBARS[10*7 + 4] = PLBARS[10*7 + 4] or 10
	
	PLBARS[10*8 + 3] = PLBARS[10*8 + 3] or 80
	-- height of 4 subbars + top/bot margin + gaps
	PLBARS[10*8 + 4] = currBar[3]*4 + currBar[1]*2 + (4-1)*currBar[2] 
	
	-- color
	local manaType = classPT[select(3, UnitClass("player"))]
	if manaType == nil then manaType = "MANA" end
	
	PLBARS[10*6 + 5] = PLBARS[10*6 + 5] or barColor[manaType]
	PLBARS[10*5 + 5] = PLBARS[10*5 + 5] or barColor["HEALTH"]
	PLBARS[10*7 + 5] = PLBARS[10*7 + 5] or barColor["CASTBAR"]
	PLBARS[10*8 + 5] = PLBARS[10*8 + 5] or barColor["SUBSTR"]
	
	-- freq
	PLBARS[6] = PLBARS[6] or 0.1
	-- all substrates
	PLBARS[5] = PLBARS[5] or barColor["SUBSTR"]
	-- swatches
	PLBARS[1] = PLBARS[1] or 40
	PLBARS[2] = PLBARS[2] or 20
	PLBARS[3] = PLBARS[3] or 12
	PLBARS[4] = PLBARS[4] or true
	-- show actionbar
	PLBARS[7] = PLBARS[7] or 1
	-- show currbar
	PLBARS[10*8 + 6] = PLBARS[10*8 + 6] or 1
	
	local centerX = GetScreenWidth()/2
	local centerY = GetScreenHeight()/2
	
	local startXX = {}
	local startXY = {}
	for i = 1, 4 do
		startXX[i] = PLBARS[10*i + 1] or centerX + 40 + (PLBARS[1] + 10) * (i - 1)
		startXY[i] = centerY - 20
	end
	startXX[5] = PLBARS[10*5 + 1] or centerX
	startXX[6] = PLBARS[10*6 + 1] or centerX
	startXX[7] = PLBARS[10*7 + 1] or centerX
	startXY[5] = centerY
	startXY[6] = centerY - PLBARS[10*5 + 4]
	startXY[7] = centerY + PLBARS[10*7 + 4]
	
	-- currency
	startXX[8] = PLBARS[10*8 + 1] or centerX
	startXY[8] = centerY + PLBARS[10*8 + 4] + 20 

	for i = 5, 8 do
		if PLBARS[10*i + 2] then 
			startXY[i] = PLBARS[10*i + 2] + PLBARS[10*i + 4]
		else
			PLBARS[10*i + 2] = startXY[i] - PLBARS[10*i + 4]
		end
	end
	for i = 1, 4 do
		if PLBARS[10*i + 2] then
			startXY[i] = PLBARS[10*i + 2] + PLBARS[2] + PLBARS[3]
		else 
			PLBARS[10*i + 2] = startXY[i] - PLBARS[2] - PLBARS[3]
		end
	end
	
	-- create all bars
	
	xRef[5][1], xRef[5][2], xRef[5][3], xRef[5][4] = 
	createFrame("HealthFrame", startXX[5], startXY[5], PLBARS[10*5 + 3], PLBARS[10*5 + 4], PLBARS[10*5 + 5], nil, true, ratio[5])
	
	xRef[6][1], xRef[6][2], xRef[6][3], xRef[6][4] = 
	createFrame("ManaFrame", startXX[6], startXY[6], PLBARS[10*6 + 3], PLBARS[10*6 + 4], PLBARS[10*6 + 5], nil, true, ratio[6])
	
	xRef[7][1], xRef[7][2], xRef[7][3], xRef[7][4], xRef[7][5] = 
	createFrame("CastbarFrame", startXX[7], startXY[7], PLBARS[10*7 + 3], PLBARS[10*7 + 4], PLBARS[10*7 + 5], " ", true, ratio[7])
	
	-- currency bars
	xRef[8][1], xRef[8][2], xRef[8][3], xRef[8][4], xRef[8][5] = 
			createFrame("CurrFrame", startXX[8], startXY[8], PLBARS[10*8 + 3], PLBARS[10*8 + 4], PLBARS[10*8 + 5])
	for i = 1, 4 do
		xRef[80+i][1], xRef[80+i][2], xRef[80+i][3], xRef[80+i][4], xRef[80+i][5] = 
			createFrame("CurrSubFrame" .. i, currBar[1], PLBARS[10*8 + 4] - currBar[1] - (currBar[3] + currBar[2])*(i - 1), 
									PLBARS[10*8 + 3] - 2*currBar[1], currBar[3], xRGBA["pitch"], " ", nil, nil, icons[i], xRef[8][1], true)
		xRef[80+i][1]:SetFrameLevel(xRef[8][1]:GetFrameLevel() + 1)
	end
	if PLBARS[10*8 + 6] == 0 then
		xRef[8][1]:Hide()
		for i = 1, 4 do
			xRef[80+i][1]:Hide()
		end
	end
	
	for i = 1, 4 do
		xRef[i][1], xRef[i][2], xRef[i][3] = 
			createSwatch("Swatch"..i, startXX[i], startXY[i], PLBARS[1], PLBARS[2], " ", " ", xRGBA["off"])
		xRef[i][4] = i
	end

	setActionBar()

	xRef[5][1]:SetScript("OnUpdate", onUpdate)
	
	print(format("|cFFffff00PlayerBars v0.5 loaded. Type /pbars -h to see command list.|r"))
	
end
end
eframe:SetScript("OnEvent", eframe.OnEvent)

local function updateSwatches()
	for i = 1, 4 do
		xRef[i][1]:SetSize(PLBARS[1], PLBARS[2])
		xRef[i][3]:SetSize(PLBARS[1], PLBARS[3])
	end
end

local cmTable = {
	["hw"] = {5,"w"},
	["hh"] = {5,"h"},
	["hc"] = {5,"c"},
	["mw"] = {6,"w"},
	["mh"] = {6,"h"},
	["mc"] = {6,"c"},
	["aw"] = {7,"w"},
	["ah"] = {7,"h"},
	["ac"] = {7,"c"}
}

SLASH_PLAYERBARS1 = "/pbars"
function SlashCmdList.PLAYERBARS(msg, editbox)
	local left, bottom, ind, type
	local cmd, args = strsplit(" ", strlower(msg), 2)
	
	if cmd == "" then
		if xRef[5][1]:IsVisible() then
			for i = 1, 8 do
				xRef[i][1]:Hide()
			end
			for i = 1, 4 do
				xRef[80+i][1]:Hide()
			end
			xRef[5][1]:SetScript("OnUpdate", nil)
		else
			for i = 1, 8 do
				xRef[i][1]:Show()
			end
			for i = 1, 4 do
				xRef[80+i][1]:Show()
			end
			xRef[5][1]:SetScript("OnUpdate", onUpdate)
		end
		return
	end
	
	if cmd == "cb" then
		if xRef[8][1]:IsVisible() then
			PLBARS[10*8 + 6] = 0
			xRef[8][1]:Hide()
			for i = 1, 4 do
				xRef[80+i][1]:Hide()
			end
		else
			PLBARS[10*8 + 6] = 1
			xRef[8][1]:Show()
			for i = 1, 4 do
				xRef[80+i][1]:Show()
			end
		end
		return
	end
	
	if cmd == "a" then
		if PLBARS[7] == 1 then
			PLBARS[7] = 0
		else
			PLBARS[7] = 1
		end
		setActionBar()
	end
	
	if cmd == "c" and args ~= "" then
		PLBARS[5] = {strsplit(" ", strlower(args), 4)}
		for i = 5, 7 do
			xRef[i][2]:SetTexture(unpack(PLBARS[5])) 
		end
		return
	end
	
	if cmd == "f" and tonumber(args) > 0 then
		PLBARS[6] = tonumber(args)
		return
	end
	
	if cmTable[cmd] ~= nil then
		ind, type = unpack(cmTable[cmd])
		if type == "w" and tonumber(args) > 0 then
			PLBARS[10*ind + 3] = tonumber(args)
			resetFrame(xRef[ind][1])
			xRef[ind][1]:SetWidth(PLBARS[10*ind + 3])
			safeSetWidth(xRef[ind][3], ratio[ind] * PLBARS[10*ind + 3])
			if ind == 7 then
				xRef[ind][5]:SetWidth(PLBARS[10*ind + 3])
			end
		elseif type == "h" and tonumber(args) > 0 then
			PLBARS[10*ind + 4] = tonumber(args)
			resetFrame(xRef[ind][1])
			xRef[ind][1]:SetHeight(PLBARS[10*ind + 4])
			xRef[ind][3]:SetHeight(PLBARS[10*ind + 4])
			if ind == 7 then
				xRef[ind][5]:SetHeight(PLBARS[10*ind + 4])
			end
			_, PLBARS[10*ind + 2] = xRef[ind][1]:GetBoundsRect()
		elseif type == "c" and args ~= "" then
			PLBARS[10*ind + 5] = {strsplit(" ", strlower(args), 4)}
			xRef[ind][4]:SetTexture(unpack(PLBARS[10*ind + 5]))
		end
		return
	end
			
	if cmd == "b" then
		left, bottom = xRef[5][1]:GetBoundsRect()
		-- align widths and pos
		for i = 6, 7 do
			PLBARS[10*i + 3] = PLBARS[10*5 + 3]
			if i == 6 then 
				xRef[i][1]:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", left, bottom)
			else
				xRef[i][1]:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", left, bottom + PLBARS[10*5 + 4] + PLBARS[10*i + 4])
			end
			resetFrame(xRef[i][1])
			xRef[i][1]:SetWidth(PLBARS[10*i + 3])
			safeSetWidth(xRef[i][3], ratio[i] * PLBARS[10*i + 3])
			PLBARS[10*i + 1], PLBARS[10*i + 2] = xRef[i][1]:GetBoundsRect()
		end
		return
	end

	if cmd == "bgh" and args ~= "" then
		xRef[1][1]:Show()
		left, bottom = xRef[1][1]:GetBoundsRect()
		for i = 2, 4 do
			xRef[i][1]:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", 
								 left + (i - 1)*(PLBARS[1] + args), 
								 bottom + PLBARS[2] + PLBARS[3])
			xRef[i][1]:Show()
			PLBARS[10 * i + 1], PLBARS[10 * i + 2] = xRef[i][1]:GetBoundsRect()
		end
		return
	end
		
	if cmd == "bgv" and args ~= "" then
		xRef[1][1]:Show()
		left, bottom = xRef[1][1]:GetBoundsRect()
		for i = 2, 4 do
			xRef[i][1]:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", 
								 left, 
								 bottom + PLBARS[2] + PLBARS[3] - (i - 1)*(PLBARS[2] + PLBARS[3] + args))
			xRef[i][1]:Show()
			PLBARS[10 * i + 1], PLBARS[10 * i + 2] = xRef[i][1]:GetBoundsRect()
		end
		return
	end
			
	if cmd == "gw" and args ~= "" then
		PLBARS[1] = args
		updateSwatches()
	elseif cmd == "gh" and args ~= "" then
		PLBARS[2] = args
		updateSwatches()
	elseif cmd == "g" then
		if xRef[1][1]:IsVisible() then
			for i = 1, 4 do
				xRef[i][1]:Hide()
			end
		else
			for i = 1, 4 do
				xRef[i][1]:Show()
			end
		end
		return
	end
		
	if cmd == "-help" or cmd == "-h" or cmd == "?" or cmd == "/?" then
		for _, v in pairs(helpText) do
			print(format("%s", v))
		end
	end
	

end



